#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : st_chat
# @Time         : 2023/8/11 14:45
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

import streamlit as st
# st.set_page_config('🔥ChatLLM', layout='wide', initial_sidebar_state='collapsed')

from openai import OpenAI
from meutils.serving.streamlit import hide_st_style, st_chat_message, ChatMessage

hide_st_style()


def gen(question):
    nesc = "\n`大模型根据公司相关法规及提问语义做答，不作为合规管理官方回复。`"

    data = {

        'model': '65f41c8a9c0ebbcbe28bb9c1',

        'messages': [
            {'role': 'user', 'content': question}
        ],
        'stream': True,

    }
    _ = OpenAI().chat.completions.create(**data)

    for chunk in _:
        if chunk.choices[0].finish_reason == 'stop':
            break
        yield chunk.choices[0].delta.content
    yield from nesc


if __name__ == '__main__':
    col1, col2, col3, *_ = st.columns(3)
    with col1:
        st.image('规丞相.png', width=64)
    with col2:
        st.markdown('##### ')
        st.markdown('##### ')
        st.markdown('##### 规丞相驾到')

    st.markdown('> ⚠️“规丞相”仅供东北证券内部测试使用，所做回答不得用于东北证券官方回复。')

    st_chat_message(ChatMessage(avatar="规丞相.png", generator="欢迎来找**规丞相**，您有什么要咨询的吗❓"))

    for message in st.session_state.messages:  # 设计his
        st_chat_message(message)

    print(st.session_state.messages)

    prompt = st.chat_input("    🤔 你可以问我任何问题", key='xx')  # 最下面
    if prompt:
        with st.spinner('AI 🤔'):
            st_chat_message(ChatMessage(name="user", generator=prompt), is_history=True)

            st_chat_message(ChatMessage(avatar="规丞相.png", generator=gen(prompt)), is_history=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2024/3/27 22:51
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import json

from meutils.pipe import *


# [**任务执行**](………………………………………….)✅
# 直接转df2md、优化超链接、
class A(BaseModel):
    id: str
    video_url: str = ''
    audio_url: str = ''
    image_url: str = 'https://cdn1.suno.ai/image_3d142559-b637-4f3a-8095-dcaaa512cc8c.png'
    image_large_url: str = 'https://cdn1.suno.ai/image_large_3d142559-b637-4f3a-8095-dcaaa512cc8c.png'
    major_model_version: str = 'v3'
    model_name: str = 'chirp-v3'
    metadata: dict = {}  # 元数据信息
    is_liked: bool = True
    user_id: str = '6275effb-094c-439a-8218-914fb33444ff'
    is_trashed: bool = True
    reaction: Optional[str] = None
    created_at: str = '2024-03-27T14:30:44.463Z'
    status: str = 'queued'
    title: str = 'Electric Dreams'
    play_count: int = 0
    upvote_count: int = 0
    is_public: bool = True


s = """
    {
        "id": "3d142559-b637-4f3a-8095-dcaaa512cc8c",
        "video_url": "",
        "audio_url": "",
        "image_url": "https://cdn1.suno.ai/image_3d142559-b637-4f3a-8095-dcaaa512cc8c.png",
        "image_large_url": "https://cdn1.suno.ai/image_large_3d142559-b637-4f3a-8095-dcaaa512cc8c.png",
        "major_model_version": "v3",
        "model_name": "chirp-v3",
        "metadata": {
            "tags": "futuristic electropop",
            "prompt": "[Verse]\nI'm standing on a rooftop, looking at the stars tonight\nMy heart is racing, can't wait for the morning light (ooh-yeah)\nI've got a feeling, something's gonna change my life\nI'm ready for the ride, can you feel it too?\n\n[Verse 2]\nThe city's buzzing, there's magic in the air\nIn every heartbeat, a dream is born and shared (ooh-yeah)\nWe're chasing rainbows, reaching for the stars above\nWe're gonna make it happen, together we'll fly high\n\n[Chorus]\nElectric dreams, they ignite the fire inside of me (oh-oh-oh)\nWith every beat, my heart skips and my spirit's free (my spirit's free)\nIn these electric dreams, we'll soar above the clouds (above the clouds)\nUnlock the possibilities, shout it out loud (shout it out loud)",
            "gpt_description_prompt": null,
            "audio_prompt_id": "8c7f666a-4df6-4657-8a83-d630b2b8ab56",
            "history": [
                {
                    "id": "8c7f666a-4df6-4657-8a83-d630b2b8ab56",
                    "continue_at": 68.04
                }
            ],
            "concat_history": null,
            "type": "gen",
            "duration": 68.04,
            "refund_credits": false,
            "stream": true,
            "error_type": null,
            "error_message": null
        },
        "is_liked": false,
        "user_id": "6275effb-094c-439a-8218-914fb33444ff",
        "is_trashed": false,
        "reaction": null,
        "created_at": "2024-03-27T14:30:44.463Z",
        "status": "queued",
        "title": "Electric Dreams",
        "play_count": 0,
        "upvote_count": null,
        "is_public": false
    }

""".strip()

print(json.loads(s))


def func(s):
    if s.endswith(".mp3"):
        return f"[🎧点击听歌]({s})"

    elif s.endswith(".mp4"):
        return f"[🖥点击观看]({s})"

    elif s.endswith(".png"):
        return f"![🖼]({s})"

    else:
        return f"```{s}```"





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : suno_api
# @Time         : 2024/4/3 16:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://suno.gcui.art/docs


from meutils.pipe import *

base_url = os.getenv("SUNO_BASE_URL")


def custom_generate_audio(payload):
    """
    {
      "prompt": "歌词",
      "tags": "pop metal male melancholic",
      "title": "歌名",
      "make_instrumental": False,
      "wait_audio": False,
    }

    :param payload:
    :return:
    """
    url = f"{base_url}/api/custom_generate"
    response = httpx.post(url, json=payload)
    return response.json()


def generate_audio_by_prompt(payload):
    """
    {
        "prompt": "A popular heavy metal song about war, sung by a deep-voiced male singer, slowly and melodiously. The lyrics depict the sorrow of people after the war.",
        "make_instrumental": False,
        "wait_audio": False
    }
    :param payload:
    :return:
    """
    url = f"{base_url}/api/generate"
    response = httpx.post(url, json=payload)
    return response.json()


def get_audio_information(audio_ids):
    url = f"{base_url}/api/get?ids={audio_ids}"
    response = httpx.get(url)
    return response.json()


def get_quota_information():
    url = f"{base_url}/api/get_limit"
    response = httpx.get(url)
    return response.json()


def song_info(df):
    """
    #   'audio_url': 'https://cdn1.suno.ai/63c85335-d8ec-4e17-882a-e51c2f358b2d.mp3',
    #   'video_url': 'https://cdn1.suno.ai/25c7e34b-6986-4f7c-a5f2-537dd80e370c.mp4',
    # https://cdn1.suno.ai/image_bea09d9e-be4a-4c27-a0bf-67c4a92d6e16.png
    :param df:
    :return:
    """
    df['🎵音乐链接'] = df['id'].map(
        lambda x: f"**请两分钟后试听**[🎧音频](https://cdn1.suno.ai/{x}.mp3)[▶️视频](https://cdn1.suno.ai/{x}.mp4)"
    )
    df['专辑图'] = df['id'].map(lambda x: f"![🖼](https://cdn1.suno.ai/image_{x}.png)")

    df_ = df[["id", "created_at", "model_name", "🎵音乐链接", "专辑图"]]

    return f"""
🎵 **「{df['title'][0]}」**

`风格: {df['tags'][0]}`

```toml
{df['prompt'][0]}
```


{df_.to_markdown(index=False).replace('|:-', '|-').replace('-:|', '-|')}
    """


from meutils.pipe import *
from meutils.cache_utils import ttl_cache
from meutils.decorators.retry import retrying
from meutils.queues.smooth_queue import SmoothQueue

from meutils.notice.feishu import send_message

from openai.types.chat.chat_completion import ChatCompletion
from openai.types.chat.chat_completion_chunk import ChatCompletionChunk

from chatllm.llmchain.completions import openai_completions
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.schemas.suno_types import SongRequest
from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk as _chat_completion_chunk

from chatllm.utils.openai_utils import to_openai_completion_params, openai_response2sse

from langchain.chat_models import ChatOpenAI
from langchain.output_parsers import OutputFixingParser, PydanticOutputParser


class Completions(object):

    def __init__(self, api_key):
        self.httpx_aclient = httpx.AsyncClient(
            base_url=api_key,
            follow_redirects=True,
            timeout=30)

    async def acreate(self, request: ChatCompletionRequest):
        chat_completion_chunk = _chat_completion_chunk.model_copy(deep=True)

        async for content in self._acreate(request):
            chat_completion_chunk.choices[0].delta.content = content
            yield chat_completion_chunk

        # 结束标识
        chat_completion_chunk.choices[0].delta.content = ""
        chat_completion_chunk.choices[0].finish_reason = "stop"
        yield chat_completion_chunk

    @retrying
    async def _acreate(self, request: ChatCompletionRequest):
        """
        Song Description ： 200
        Lyrics：1250
        Style of Music：120
        Title：80
        """

        prompt = request.messages[-1]["content"]
        send_message(prompt)
        if prompt.startswith(("使用四到五个字直接返回这句话的简要主题",
                              "简要总结一下对话内容，用作后续的上下文提示 prompt，控制在 200 字以内")):
            return

        if len(prompt.strip()) < 100:
            yield "> Custom Mode ❎: 生成歌词\n\n"
            payload = {
                "gpt_description_prompt": prompt,
                "mv": "chirp-v3-0",
                "prompt": ""
            }
            df = await self.generate_by_prompt(payload)
        else:
            yield "> Custom Mode ✅：解析歌词\n\n"
            song_request: SongRequest = self.prompt_parser(prompt)  # SongRequest
            payload = {
                "title": song_request.title,
                "tags": song_request.music_style,
                "prompt": song_request.lyrics,
                "continue_at": song_request.continue_at,
                "continue_clip_id": song_request.continue_clip_id,

                "mv": "chirp-v3-0",

            }

            # title='My Song' lyrics='[Verse]...[Chorus]...[Bridge]...' music_style='Pop' continue_clip_id='8c7f666a-4df6-4657-8a83-d630b2b8ab56' continue_at=120
            # {
            #   "prompt": "string",
            #   "mv": "chirp-v3-0",
            #   "title": "string",
            #   "tags": "string",
            #   "continue_at": 120,
            #   "continue_clip_id": "string"
            # }
            send_message(str(payload))
            yield f"""```json\n{payload}\n```"""
            df = await self.custom_generate(payload)

        ids = ','.join(df['id'].tolist())  # df 为空

        yield f"""```\nTask ids: {ids}\n```\n\n"""
        yield f"""[音乐任务]("""

        for i in range(100):
            yield f"""{'🎵' if i % 2 else '🔥'}"""
            await asyncio.sleep(1)

            if i > 10:  # 5秒后才开始回调
                await asyncio.sleep(3)
                df = await self.get_information(ids)

                logger.debug("回调歌曲")

                if all(df.status == 'streaming'):  # 歌词生成
                    yield f""") ✅\n\n"""
                    _ = song_info(df)
                    yield _
                    send_message(_)
                    break
                elif all(df.status == 'error'):
                    yield f""") ❎\n\n"""
                    yield f"""⚠️触发内容审查，请修改后重试"""
                    send_message(df.to_markdown())
                    break

    def create_sse(self, request: ChatCompletionRequest):
        return openai_response2sse(self.acreate(request), redirect_model=request.model)

    @retrying
    async def custom_generate(self, payload):
        response = await self.httpx_aclient.post("/generate", json=payload)
        return pd.DataFrame(response.json().get('clips'))

    @retrying(min_seconds=3)
    async def generate_by_prompt(self, payload):
        response = await self.httpx_aclient.post("/generate/description-mode", json=payload)
        # print(response.json())
        return pd.DataFrame(response.json().get('clips'))

    @retrying
    async def get_information(self, ids):
        response0 = await self.httpx_aclient.get(f"/feed/{ids.split(',')[0]}")
        response1 = await self.httpx_aclient.get(f"/feed/{ids.split(',')[1]}")

        clips = response0.json() + response1.json()

        return pd.DataFrame([{**clip, **clip['metadata']} for clip in clips])

    def prompt_parser(self, prompt):
        song_parser = OutputFixingParser.from_llm(
            parser=PydanticOutputParser(pydantic_object=SongRequest),
            llm=ChatOpenAI(temperature=0)
        )
        result = song_parser.parse(completion=prompt)  # 错误被自动修正
        return result


if __name__ == '__main__':
    prompt = """
    在10s处

    继续创作

    "6e579766-08ec-4b47-8674-1ebd6f3a366a"

    """
    prompt += """
        请根据我的自定义歌词和音乐风格来创作歌曲：
        中文标题：酒入愁肠
        英文标题：Wine and Sorrows

        歌词结构：
        [Intro]
        （独奏）

        [Verse 1]
        月光洒在破旧的篱笆，
        李白提壶独自彷徨。
        酒香飘逸入云端，
        笔下是豪迈的想象。

        [Chorus]
        一壶浊酒喜相逢，
        千载孤愁只杜甫懂。
        古道西风瘦马间，
        人间事，两行清泪长。

        [Verse 2]
        江水流淌带走忧伤，
        杜甫凝眉思国家。
        文章锋利剑如霜，
        心中苦，却言者无罪当。

        [Chorus]
        一壶浊酒喜相逢，
        千载孤愁只杜甫懂。
        古道西风瘦马间，
        人间事，两行清泪长。

        [Bridge]
        一千年来谁能懂，
        酒与愁是最深的重。
        历史长河波澜壮，
        诗人眼中看尽繁华空。

        [Chorus]
        一壶浊酒喜相逢，
        千载孤愁只杜甫懂。
        古道西风瘦马间，
        人间事，两行清泪长。

        [Outro]
        （独奏）

        音乐风格：
        传统民谣
        主要乐器：古筝、琵琶、笛子
        节奏：缓慢而深情
        氛围：沉郁、古典、充满历史感
    """
    # print(Completions('http://154.3.0.117:39955').prompt_parser(prompt))

    # _ = arun(Completions('http://154.3.0.117:39955').generate_by_prompt({"gpt_description_prompt": "写首歌曲"}))
    # print(_)
    payload = {
        "title": "Wine and Sorrows",
        "tags": "传统民谣",
        "prompt": "[Intro]\n（独奏）\n\n[Verse 1]\n月光洒在破旧的篱笆，\n李白提壶独自彷徨。\n酒香飘逸入云端，\n笔下是豪迈的想象。\n\n[Chorus]\n一壶浊酒喜相逢，\n千载孤愁只杜甫懂。\n古道西风瘦马间，\n人间事，两行清泪长。\n\n[Verse 2]\n江水流淌带走忧伤，\n杜甫凝眉思国家。\n文章锋利剑如霜，\n心中苦，却言者无罪当。\n\n[Chorus]\n一壶浊酒喜相逢，\n千载孤愁只杜甫懂。\n古道西风瘦马间，\n人间事，两行清泪长。\n\n[Bridge]\n一千年来谁能懂，\n酒与愁是最深的重。\n历史长河波澜壮，\n诗人眼中看尽繁华空。\n\n[Chorus]\n一壶浊酒喜相逢，\n千载孤愁只杜甫懂。\n古道西风瘦马间，\n人间事，两行清泪长。\n\n[Outro]\n（独奏）",
        "continue_at": 120,
        # "continue_clip_id": None,
        "mv": "chirp-v3-0"
    }
    _ = arun(Completions('http://154.3.0.117:39955').custom_generate(payload))
    print(_)

#     data = generate_audio_by_prompt({
#         "prompt": "A popular heavy metal song about war, sung by a deep-voiced male singer, slowly and melodiously. The lyrics depict the sorrow of people after the war.",
#         "make_instrumental": False,
#         "wait_audio": False
#     })
#
#     # ids = f"{data[0]['id']},{data[1]['id']}"
#     # print(f"ids: {ids}")
#     #
#     # for _ in range(60):
#     #     data = get_audio_information(ids)
#     #     if data[0]["status"] == 'streaming':
#     #         print(f"{data[0]['id']} ==> {data[0]['audio_url']}")
#     #         print(f"{data[1]['id']} ==> {data[1]['audio_url']}")
#     #         break
#     #     # sleep 5s
#     #     time.sleep(5)
#
#     payload = {
#         "prompt": "我不是李白" * 100,
#         "tags": "pop metal male melancholic",
#         "title": "歌名",
#         # "make_instrumental": False,
#         # "wait_audio": False,
#     }
#
#     print(custom_generate_audio(payload))
# # {'error': 'Prompt, tags, and title are required'}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : completions
# @Time         : 2023/12/19 16:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 逆向工程
# todo: llm2gpt

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from sse_starlette import EventSourceResponse
from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status

from openai import OpenAI, AsyncOpenAI
from openai.types.chat import ChatCompletion, ChatCompletionChunk

from chatllm.llmchain.completions import kimi
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import openai_response2sse

router = APIRouter()

ChatCompletionResponse = Union[ChatCompletion, List[ChatCompletionChunk]]


@router.post("/chat/completions")
async def create_chat_completions(
    request: ChatCompletionRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    logger.debug(request)

    api_key = auth and auth.credentials or None
    if api_key is None:
        detail = {
            "error": {
                "message": "",
                "type": "invalid_request_error",
                "param": None,
                "code": "invalid_api_key",
            }
        }
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail=detail)

    # 轮询
    api_keys = api_key.strip().split('|')
    api_key = api_keys[0] if len(api_keys) == 1 else np.random.choice(api_keys)

    # kimi kimi-128k kimi-all rag-kimi
    # moonshot-v1-8k moonshot-v1-32k moonshot-v1-128k
    if request.model == "kimi-all" and request.messages[-1]["content"].strip().startswith(("画",)):
        # 增加画图功能
        response = await AsyncOpenAI().chat.completions.create(
            model="gpt-4-mobile",
            messages=request.messages[-1:],
            stream=True
        )
        return openai_response2sse(response, redirect_model=request.model)

    if request.file_ids or request.model == 'kimi-me':  # todo: 根据fileid映射对应的refresh_token
        # 增加计费
        request.use_search = False
        completions = kimi.Completions()

    elif request.model.startswith(("moonshot",)):
        request.use_search = False
        completions = kimi.Completions(api_key=api_key)
    else:
        completions = kimi.Completions(api_key=api_key)

    response = completions.create_sse(request)
    return response


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/v1')

    app.run()
    # for i in range(10):
    #     send_message(f"兜底模型", title="github_copilot")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : suno_types
# @Time         : 2024/3/28 19:21
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *


class SongRequest(BaseModel):
    title: str = Field(description="song title")
    lyrics: str = Field(description="Enter lyrics, example: [Intro]...[Verse]...[Chorus]...")
    music_style: str = Field(description="the Style of Music, Maximum 10 words", examples=['syncopated country ...'])
    continue_clip_id: Optional[str] = Field(
        description="Continue creating based on song id/clip_id",
        examples=['8c7f666a-4df6-4657-8a83-d630b2b8ab56']
    )
    continue_at: Optional[int] = Field(120, description="Continue creating based on a certain point in time, for example at 10s")


class Response(BaseModel):
    code: Optional[int] = 0
    msg: Optional[str] = "success"
    data: Optional[Any] = None


class GenerateBase(BaseModel):
    prompt: str = ""
    mv: str = "chirp-v3"
    title: str = ""
    tags: str = ""
    continue_at: Optional[str] = None
    continue_clip_id: Optional[str] = None


class SunoRequest(BaseModel):
    """
    Welcome to Custom Mode 欢迎使用自定义模式

    Start with Lyrics: Write down your thoughts, or hit “Make Random Lyrics” for spontaneous creativity. Prefer no words? Opt for “Instrumental” and let the tunes express themselves.
    从歌词开始写下你的想法，或点击 "制作随机歌词 "进行即兴创作。不喜欢歌词？选择 "乐器"，让曲调来表达自己。

    Choose a Style: Select your “Style of Music” to set the vibe, mood, tempo and voice. Not sure? “Use Random Style” might surprise you with the perfect genre.
    选择风格：选择您的 "音乐风格"，设定氛围、情绪、节奏和声音。不确定？"使用随机风格 "可能会让你惊喜地发现完美的音乐风格。

    Extend Your Song: Want to go longer? Use the more actions (…) menu, select "Continue From This Song", select the desired time to extend your song from, and press Create. Use “Get Full Song” to put the full song together.
    延长您的歌曲：想延长时间？使用 "更多操作"（...）菜单，选择 "从这首歌开始继续"，选择想要延长歌曲的时间，然后按 "创建"。使用 "获取完整歌曲 "将完整歌曲放在一起。

    Unleash Your Creativity: Dive into the world of music-making and let your imagination run wild. Happy composing! 🎉
    释放你的创造力：潜入音乐创作世界，尽情发挥你的想象力。祝您创作愉快🎉
    """

    song_description: str = ""  # '一首关于在雨天寻找爱情的富有感染力的朋克歌曲' todo: gpt润色
    """
        Describe the style of music and topic youwant (e.g. acoustic pop about theholidays).
        Use genres and vibes insteadof specific artists and songs
    """

    instrumental: bool = False
    """创作一首没有歌词的歌曲。"""

    custom_mode: bool = True
    """Suno 专为创作原创音乐而设计。请确认您只提交人工智能生成的歌词、原创歌词或您有权继续使用的歌词。"""

    title: str = ''

    music_style: str = "R&B and soul"  # 可随机
    tags: str = music_style

    mv: str = 'chirp-v3-0'  # 模型

    lyrics: Optional[str] = None  # 自动生成
    prompt: Optional[str] = None  # 自动生成
    """
        [Verse]
        Wake up in the morning, feeling kind of tired
        Rub my eyes, stretch my limbs, try to get inspired
        Open up the cupboard, see that shiny little jar
        It's my secret weapon, helps me reach the stars

        [Verse 2]
        Fill my favorite mug with that dark and steamy brew
        Inhale the aroma, it's my daily rendezvous
        Sip it nice and slow, feel the warmth flow through my veins
        Oh, coffee in the morning, you're my sugar, you're my dreams

        [Chorus]
        Coffee in the morning, you're my lifeline, can't deny
        You bring me energy when the day is looking gray
        Coffee in the morning, you're my sunshine in a cup
        With every sip, I'm feeling alive, ready to seize the day
    """
    # 继续创作
    continue_at: Optional[float] = None
    continue_clip_id: Optional[str] = None  # "8c7f666a-4df6-4657-8a83-d630b2b8ab56"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.prompt = self.song_description


if __name__ == '__main__':
    print(SunoRequest(song_description='一首关于在雨天寻找爱情的富有感染力的朋克歌曲'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatdoc
# @Time         : 2023/7/15 20:53
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


from meutils.pipe import *

from chatllm.llmchain.utils import docs2dataframe
from chatllm.llmchain.decorators import llm_stream
from chatllm.llmchain.vectorstores import Milvus
from chatllm.llmchain.embeddings import OpenAIEmbeddings
from chatllm.llmchain.document_loaders import FilesLoader

from langchain.text_splitter import *
from langchain_community.chat_models import ChatOpenAI
from langchain.chains.question_answering import load_qa_chain
from langchain.embeddings.base import Embeddings


class ChatBase(object):

    def __init__(self,
                 model: str = "gpt-3.5-turbo-16k-0613", temperature: float = 0,
                 collection_name: str = 'TEST', chunk_size: int = 5,
                 get_api_key: Optional[Callable[[int], List[str]]] = None,
                 ):
        self.collection_name = collection_name
        # 根据 model 选择 llm/openai_api_base
        self.llm = ChatOpenAI(model=model, temperature=np.clip(temperature, 0, 1), streaming=True)
        self.embeddings = OpenAIEmbeddings(chunk_size=chunk_size)  # todo: 可选
        if get_api_key:
            self.embeddings.get_api_key = get_api_key
            self.embeddings.openai_api_key = get_api_key(1)[0]
            self.llm.openai_api_key = get_api_key(1)[0]

        self.chain = load_qa_chain(self.llm, chain_type="stuff")

        # self.chain.llm_chain.prompt.messages[0].prompt.template = \
        #     f'System time: {datetime.datetime.now()};' \
        #     + self.chain.llm_chain.prompt.messages[0].prompt.template

        _vdb_kwargs = self.vdb_kwargs.copy()
        _vdb_kwargs['embedding_function'] = _vdb_kwargs.pop('embedding')
        self.vdb: Optional[Milvus] = Milvus(**_vdb_kwargs)

    def pipeline(self):
        pass

    def llm_qa(self, query: str, k: int = 5, threshold: float = 0.5, **kwargs: Any):
        """todo: pipeline"""
        docs = self.vdb.similarity_search(query, k=max(k, 10), threshold=threshold, **kwargs)
        docs = docs | xUnique_plus(lambda doc: doc.page_content.strip())  # 按内容去重，todo: 按语义相似度去重
        docs = docs[:k]
        # todo: 上下文信息
        docs = [Document(page_content=f"system time: {datetime.datetime.now()}")] + docs
        if docs:
            # chain.run(input_documents=docs, question=query)
            return llm_stream(self.chain.run)({"input_documents": docs, "question": query})

    @staticmethod
    def load_file(file_paths, chunk_size=2000, chunk_overlap=200, **kwargs):
        """支持多文件"""
        loader = FilesLoader(file_paths)
        docs = loader.load_and_split(
            RecursiveCharacterTextSplitter(chunk_size=chunk_size, chunk_overlap=chunk_overlap, add_start_index=True)
        )
        return docs

    @diskcache(location=os.getenv('INSERT_VECTOR_CACHE', '~/.cache/insert_vector_cache'), ignore=['self'])
    def create_index(self, docs: List[Document], **kwargs):
        """初始化 drop_old=True"""
        self.vdb = Milvus.from_documents(docs, **{**self.vdb_kwargs, **kwargs})

    @cached_property
    def vdb_kwargs(self):
        connection_args = {
            'uri': os.getenv('ZILLIZ_ENDPOINT'),
            'token': os.getenv('ZILLIZ_TOKEN')
        }
        address = os.getenv('MILVUS_ADDRESS')  # 该参数优先
        if address:
            connection_args.pop('uri')
            connection_args['address'] = address

        index_params = {
            "metric_type": "IP",
            "index_type": "IVF_FLAT",
            "params": {"nlist": 1024}
        }

        embedding_function = self.embeddings

        vdb_kwargs = dict(
            embedding=embedding_function,
            connection_args=connection_args,
            index_params=index_params,
            search_params=None,
            collection_name=self.collection_name,
            drop_old=False,
        )

        return vdb_kwargs



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chat_multimodal
# @Time         : 2024/4/22 13:36
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
"""
结构体通用化转发

"""

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.str_utils.regular_expression import parse_url

from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import openai_response2sse, to_openai_completion_params

from openai import OpenAI, AsyncOpenAI


class Completions(object):
    def __init__(self, api_key=None):
        params = dict(
            api_key=api_key,
            base_url='https://api.chatfire.cn/v1',
            max_retries=1,
        )
        # 如果这一步就报错呢
        self.client = AsyncOpenAI(**params)

    def create(self, request: ChatCompletionRequest):
        logger.debug(str(request.messages))

        prompt = request.messages[-1].get('content')
        request.messages[-1]['content'] = self.url2part_content(prompt)
        data = to_openai_completion_params(request)

        logger.debug(request.messages)

        return OpenAI().chat.completions.create(**data)

    async def acreate(self, request: ChatCompletionRequest, **kwargs):
        logger.debug(str(request.messages))

        prompt = request.messages[-1].get('content')
        request.messages[-1]['content'] = self.url2part_content(prompt)
        data = to_openai_completion_params(request)

        logger.debug(str(request.messages))

        _ = await self.client.chat.completions.create(**data)

        return _

    @staticmethod
    def url2part_content(content):  # todo: 文件url无法放image_url里
        """几种处理方式
        1. url放入[{"type": "image_url", "image_url": {"url": ""}}]
        2. 根据url提取出内容 [content](url)  待确认
        """
        # part content
        # if isinstance(content, list):  # [{'role': 'user', 'content':  [{"type": "text", "text": ""}]]
        #     for part in content:
        #         if part.get("type")==""

        # content = f"{content}\n```url\n{reader(url)}\n```"

        if isinstance(content, str):  # 当前针对text，单轮
            text = content
            urls = parse_url(text, for_image=True)

            content = [
                {"type": "text", "text": text},
                *[{"type": "image_url", "image_url": {"url": url}} for url in urls]
            ]
        elif isinstance(content, list):  # 识图标准化
            parts = []
            for part in content:
                if part.get("type") == "image_url" and isinstance(part.get("image_url"), str):
                    # {'type': 'image_url', 'image_url': 'http://ai.chatfire.cn/files/images/xx.jpg'}
                    part["image_url"] = {"url": part["image_url"]}
                parts.append(part)

        return content


if __name__ == '__main__':
    messages = [
        {
            'role': 'user',
            'content': "解释下这张照片https://img-home.csdnimg.cn/images/20201124032511.png"
        },
    ]

    # messages = [
    #     {
    #         'role': 'user',
    #         'content': [{"type": "text", "text": "解释下这张照片 https://img-home.csdnimg.cn/images/20201124032511.png"}]
    #     }
    # ]

    messages = [
        {
            'role': 'user',
            'content': [
                {'type': 'text', 'text': '解释下这张照片'},
                {'type': 'image_url', 'image_url': 'https://img-home.csdnimg.cn/images/20201124032511.png'}] # 不标准
        }
    ]

    request = ChatCompletionRequest(model="step-1", messages=messages)  # 针对all
    print(Completions().create(request))

    # async def main():
    #     async for i in Completions().acreate(request):
    #         print(i)
    #
    # arun(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : kimi
# @Time         : 2024/2/29 15:09
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/search?q=https%3A%2F%2Fkimi.moonshot.cn%2Fapi%2Fchat&type=code

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.str_utils.regular_expression import parse_url

from chatllm.schemas.kimi_types import KimiData
from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import openai_response2sse

from fastapi import UploadFile
from openai.types.file_object import FileObject


class Completions(object):
    def __init__(self, **client_params):
        self.api_key = client_params.get('api_key')  # refresh_token
        self.access_token = client_params.get("access_token") or self.get_access_token(self.api_key)  # access_token

        self.httpx_client = httpx.Client(headers=self.headers, follow_redirects=True)
        self.httpx_aclient = httpx.AsyncClient(headers=self.headers, follow_redirects=True)

    def create(self, request: ChatCompletionRequest):
        request = self.do_request(request)

        url = f"https://kimi.moonshot.cn/api/chat/{self.chat_id}/completion/stream"

        payload = request.model_dump()
        # response = self.httpx_client.post(url=url, json=payload)
        response: httpx.Response
        with self.httpx_client.stream("POST", url=url, json=payload) as response:
            for line in response.iter_lines():
                yield from self.do_chunk(line)

    async def acreate(self, request: ChatCompletionRequest):
        request = self.do_request(request)

        # chat_id =request.conversation_id or self.chat_id # 会话失败还得重新建

        url = f"https://kimi.moonshot.cn/api/chat/{self.chat_id}/completion/stream"

        payload = request.model_dump()
        # response = self.httpx_client.post(url=url, json=payload)
        response: httpx.Response
        async with self.httpx_aclient.stream("POST", url=url, json=payload) as response:
            async for line in response.aiter_lines():
                for chunk in self.do_chunk(line):
                    yield chunk

                    # 多输出几行就可以替换掉kimi【水印】
                    # 我是Kimi，由月之暗面科技有限公司开发的人工智能助手。我在这里为您提供帮助，回答问题，并协助您处理各种信息和任务。如果您有任何问题或需要帮助，请随时告诉我！

    def create_sse(self, request: ChatCompletionRequest):
        if request.stream:
            return openai_response2sse(self.acreate(request), redirect_model=request.model)
        else:
            _chat_completion = chat_completion.model_copy(deep=True)
            _chat_completion.usage.prompt_tokens = len(request.messages[-1]["content"])
            for i in self.create(request):
                _chat_completion.choices[0].message.content += i.choices[0].delta.content

            return openai_response2sse(_chat_completion, redirect_model=request.model)

    def do_request(self, request: ChatCompletionRequest):

        history = request.messages[:-1]
        history = history and f"""可参考系统历史对话回答问题：\n```json\n{json.dumps(history, ensure_ascii=False, indent=4)}\n```"""  # todo: 优化
        # history = history and f"""History:\n```json\n{json.dumps(history, ensure_ascii=False, indent=4)}\n```"""

        question = request.messages[-1]["content"]
        if isinstance(question, list):
            # 文件  {"type": "image_url", "image_url": {"url": image_url1}}
            # {
            #     "type": "http://ai.chatfire.cn/files/document/绩效面谈表-模版-nine-1710139239100-nine-66b3829d5.pdf text"
            # }

            send_message(request.messages)

        # 链接转换
        if not request.model.startswith(("moonshot",)):  # moonshot不支持链接
            question = self._url2content(question) or question

        request.messages = [
            {
                'role': 'user',
                'content': f"""{history}\n\nQuery: {question}""" if history else question
            }
        ]

        return request

    def do_chunk(self, line):

        if line := line.strip().strip('data: '):

            logger.debug(line)

            kimi_data = KimiData.model_validate_json(line)

            for chunk in self.search_plus(kimi_data):
                yield chunk

            if kimi_data.event == 'cmpl':
                chat_completion_chunk.choices[0].delta.content = kimi_data.content
                yield chat_completion_chunk

            if kimi_data.event == 'all_done':
                _ = chat_completion_chunk.model_copy(deep=True)
                _.choices[0].delta.content = ""
                _.choices[0].finish_reason = "stop"  # 特殊
                yield _
                return

    def _url2content(self, content) -> Union[bool, str]:
        urls = parse_url(content)
        if not urls: return False

        for i, url in enumerate(urls):
            _url = f""" <url id="{i}" type="url" status="" title="" wc="">{url}</url> """
            content = content.replace(url, _url)

        return content

    def search_plus(self, kimi_data: KimiData):
        if kimi_data.event == 'search_plus':
            if kimi_data.msg.get("type") == "start":  # start_res
                chat_completion_chunk.choices[0].delta.content = "---\n🔎 开始搜索 🚀\n"
                yield chat_completion_chunk
            if kimi_data.msg.get("type") == "get_res":
                title = kimi_data.msg.get("title")
                url = kimi_data.msg.get("url")
                chat_completion_chunk.choices[0].delta.content = f"""- 🔗 [{title}]({url})\n"""
                yield chat_completion_chunk

            if kimi_data.msg.get("type") == "answer":
                chat_completion_chunk.choices[0].delta.content = f"""---\n\n"""
                yield chat_completion_chunk

    @property
    def chat_id(self):
        url = "https://kimi.moonshot.cn/api/chat"
        payload = {"name": str(datetime.datetime.now()), "is_example": False}
        response = self.httpx_client.post(url, json=payload)

        if response.status_code != 200:
            send_message(f"Kimi access_token:\n\n{response.text}", title="Kimi")
            response.raise_for_status()

        return response.json().get('id')

    @property
    def headers(self):
        return {
            'Authorization': f"Bearer {self.access_token}",
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36'
        }

    @staticmethod
    @ttl_cache(600)
    def get_access_token(refresh_token=None):  # 设计重试
        refresh_token = refresh_token or os.getenv(
            "KIMI_REFRESH_TOKEN") or "eyJhbGciOiJIUzUxMiIsInR5cCI6IkpXVCJ9.eyJpc3MiOiJ1c2VyLWNlbnRlciIsImV4cCI6MTcxNjk2NzcxNiwiaWF0IjoxNzA5MTkxNzE2LCJqdGkiOiJjbmczNDkybG5sOTB2cnIzY21qZyIsInR5cCI6InJlZnJlc2giLCJzdWIiOiJja2kwOTRiM2Flc2xnbGo2Zm8zMCIsInNwYWNlX2lkIjoiY2tpMDk0YjNhZXNsZ2xqNmZvMmciLCJhYnN0cmFjdF91c2VyX2lkIjoiY2tpMDk0YjNhZXNsZ2xqNmZvMzAifQ.S2T2c3rfFaQmyYMURLpgpmp2O1Voojy3b6-qoP0Hnrlvk6Y8Zxn2ku6U0ZEMW48KbG-fqaYlbF8lWUfsuSVSEQ"

        headers = {
            'Authorization': f"Bearer {refresh_token}",
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36'
        }
        url = "https://kimi.moonshot.cn/api/auth/token/refresh"
        response = httpx.get(url, headers=headers, verify=False)

        # logger.debug(refresh_token)
        # logger.debug(response.text)
        # logger.debug(response.status_code)

        if response.status_code != 200:
            send_message(f"Kimi refresh_token:\n\n{response.text}\n\n{refresh_token}", title="Kimi")
            response.raise_for_status()

        # refresh_token = response.get("refresh_token") # 是否去更新
        return response.json().get("access_token")

    def file_extract(self, file: Union[str, Path, UploadFile]):

        if isinstance(file, str) and Path(file).exists():
            filename = Path(file).name
            file = Path(file).read_bytes()
        else:
            filename = file.filename or file.file.name
            file = file.file

        url = "https://kimi.moonshot.cn/api/pre-sign-url"
        payload = {"action": "file", "name": filename}
        pre_sign_response = self.httpx_client.post(url, json=payload).json()

        # logger.debug(pre_sign_response)

        url = pre_sign_response.get('url')
        object_name = pre_sign_response.get("object_name")

        # 上传文件
        files = {'file': file}
        self.httpx_client.put(url, files=files)

        # 获取文件id
        url = "https://kimi.moonshot.cn/api/file"
        payload = {"type": "file", "name": filename, "object_name": object_name}
        file_response = self.httpx_client.post(url, json=payload, headers=self.headers).json()

        # logger.debug(file_response)
        # {
        #     "id": "cnrtn82lnl9f8h6k6eqg",
        #     "name": "东北证券股份有限公司合规手册（东证合规发〔2023〕22号 20231228）.pdf",
        #     "parent_path": "",
        #     "type": "file",
        #     "status": "initialized",
        #     "presigned_url": "https://prod-chat-kimi.tos-s3-cn-beijing.volces.com/prod-chat-kimi/cki094b3aeslglj6fo2g/2024-03-18/cnrtn7998onq6ckol4jg/%E4%B8%9C%E5%8C%97%E8%AF%81%E5%88%B8%E8%82%A1%E4%BB%BD%E6%9C%89%E9%99%90%E5%85%AC%E5%8F%B8%E5%90%88%E8%A7%84%E6%89%8B%E5%86%8C%EF%BC%88%E4%B8%9C%E8%AF%81%E5%90%88%E8%A7%84%E5%8F%91%E3%80%942023%E3%80%9522%E5%8F%B7%2020231228%EF%BC%89.pdf?X-Amz-Algorithm=AWS4-HMAC-SHA256\u0026X-Amz-Credential=AKLTYTJlNjgwMjY2ZDBkNDFiYmI5YWNiZDBlZmFmYjIzZTA%2F20240318%2Fcn-beijing%2Fs3%2Faws4_request\u0026X-Amz-Date=20240318T061352Z\u0026X-Amz-Expires=3600\u0026X-Amz-SignedHeaders=host\u0026X-Amz-Signature=95041dfc608a81e07fe0b91d6bb44668112d057b6b6313dde6d23ea1e76c9b77",
        #     "text_presigned_url": "https://prod-chat-kimi.tos-s3-cn-beijing.volces.com/prod-chat-kimi/cki094b3aeslglj6fo2g/2024-03-18/cnrtn82lnl9f8h6k6eq0/?X-Amz-Algorithm=AWS4-HMAC-SHA256\u0026X-Amz-Credential=AKLTYTJlNjgwMjY2ZDBkNDFiYmI5YWNiZDBlZmFmYjIzZTA%2F20240318%2Fcn-beijing%2Fs3%2Faws4_request\u0026X-Amz-Date=20240318T061352Z\u0026X-Amz-Expires=3600\u0026X-Amz-SignedHeaders=host\u0026X-Amz-Signature=c1157fd8a0b65864a89480bb9e743d48004705126a7c04c24c0e2abc15131773",
        #     "uploaded_at": "2024-03-18T06:13:49Z",
        #     "created_at": "2024-03-18T14:13:52.036471136+08:00",
        #     "updated_at": "2024-03-18T14:13:52.036471136+08:00"
        # }

        file_id = file_response.get("id")

        # 解析文件
        url = "https://kimi.moonshot.cn/api/file/parse_process"
        payload = {"ids": [file_id]}
        resp = self.httpx_client.post(url, json=payload, headers=self.headers, timeout=180).text.strip('data: ')
        resp = json.loads(resp)
        resp['file_info'].update(file_response)

        logger.debug(resp)

        status = resp.get("status") == "parsed" and "processed" or "error"

        file_object = FileObject(
            id=file_id,
            bytes=resp['file_info']['size'],
            created_at=int(time.time()),
            filename=filename,
            object='file',
            purpose="assistants",
            status=status,
            status_details=str(resp)
        )

        return file_object

    def file_upload(self, file: Union[str, UploadFile]):
        return self.file_extract(file)


# 2024-03-04 09:29:24.815 | DEBUG    | __main__:f:186 - {'ids': ['cnii7t4udu62fbesjteg']}
# data: {"event":"resp","file_info":{"id":"cnii7t4udu62fbesjteg","name":"å­å­åµæ³.pdf","type":"file","content_type":"pdf","status":"parsed","size":607093,"token_size":91753,"failed_reason":""},"id":"cnii7t4udu62fbesjteg","status":"parsed"}


# def do_refresh_token(refresh_token):
#     headers = {
#         'Authorization': f'Bearer {refresh_token}'
#     }
#
#     response = requests.get("https://kimi.moonshot.cn/api/auth/token/refresh", headers=headers)
#     print(response.status_code)
#
#     return response.json()  # {"access_token": "", "refresh_token": ""} {'error_type': 'auth.token.invalid', 'message': '您的授权已过期，请重新登录'}
#
#
# refresh_token = "eyJhbGciOiJIUzUxMiIsInR5cCI6IkpXVCJ9.eyJpc3MiOiJ1c2VyLWNlbnRlciIsImV4cCI6MTcxNjk2NzcxNiwiaWF0IjoxNzA5MTkxNzE2LCJqdGkiOiJjbmczNDkybG5sOTB2cnIzY21qZyIsInR5cCI6InJlZnJlc2giLCJzdWIiOiJja2kwOTRiM2Flc2xnbGo2Zm8zMCIsInNwYWNlX2lkIjoiY2tpMDk0YjNhZXNsZ2xqNmZvMmciLCJhYnN0cmFjdF91c2VyX2lkIjoiY2tpMDk0YjNhZXNsZ2xqNmZvMzAifQ.S2T2c3rfFaQmyYMURLpgpmp2O1Voojy3b6-qoP0Hnrlvk6Y8Zxn2ku6U0ZEMW48KbG-fqaYlbF8lWUfsuSVSEQ"
#
# with timer("监控token过期时间"):
#     for i in tqdm(range(1000)):
#         response = do_refresh_token(refresh_token)
#         refresh_token = response.get("refresh_token")
#         access_token = response.get("access_token")
#         headers = {
#             'authorization': f'Bearer {access_token}'
#         }
#         response = requests.post(
#             "https://kimi.moonshot.cn/api/chat/cng35i6cp7f94p55g2u0/completion/stream",
#             json={'messages': [{'role': 'user', 'content': '1+1'}]},
#             headers=headers,
#
#         )
#         response.encoding = 'utf8'
#
#         logger.debug(response.text)
#
#         if not refresh_token:
#             break
#         time.sleep(60)


if __name__ == '__main__':
    # print(Completions.get_access_token())
    # print(Completions().access_token)

    # print(Completions().chat_id)
    t = "eyJhbGciOiJIUzUxMiIsInR5cCI6IkpXVCJ9.eyJpc3MiOiJ1c2VyLWNlbnRlciIsImV4cCI6MTcwOTgwMDIxMCwiaWF0IjoxNzA5Nzk5MzEwLCJqdGkiOiJjbmtuZjNocmRpamFpbGlkNGNqZyIsInR5cCI6ImFjY2VzcyIsInN1YiI6ImNraTA5NGIzYWVzbGdsajZmbzMwIiwic3BhY2VfaWQiOiJja2kwOTRiM2Flc2xnbGo2Zm8yZyIsImFic3RyYWN0X3VzZXJfaWQiOiJja2kwOTRiM2Flc2xnbGo2Zm8zMCJ9.fm5Hd16J3DNtmRT4zctcKrIvojEcByORFfHm2OziddOMF1dL5nqX3U9skERvF-q-QlpSpMSm8SDRVLXde1Balw"
    c = Completions(access_token=t)
    c = Completions()

    # async def f(x):
    #     _ = c.create(
    #         ChatCompletionRequest(
    #             messages=[
    #                 # {'role': 'user', 'name': '_resource', 'content': "你现在扮演的角色是GPT请牢记"},
    #                 {'role': 'user', 'content': '1+1'},  # 不能有系统信息
    #                 # {'role': 'user', 'content': '总结这篇文章 https://f.chatllm.vip/rag-dev/trento2013.pdf'},
    #                 # {'role': 'user', 'content': '总结这篇文章'},
    #
    #             ],
    #             # file_ids=['cnijpe6cp7f0sq79ahtg']
    #             # file_ids=['cnimvq03r076lsmj7ohg']
    #         )
    #     )
    #
    #     _ = c.acreate(
    #         ChatCompletionRequest(
    #             messages=[
    #                 {'role': 'user', 'content': '1+1'},  # 不能有系统信息
    #
    #             ],
    #
    #         )
    #     )
    #     s = ""
    #     async for i in _:
    #         s += i.choices[0].delta.content
    #         print(i.choices[0].delta.content, end='')
    #
    #     chat_completion.choices[0].message.content = s
    #     return chat_completion
    #
    #
    # print(arun(f('')))

    # with timer("过期时间"):
    #     while 1:
    #
    #         try:
    #
    #             f('')
    #
    #             time.sleep(10)
    #
    #         except Exception as e:
    #             print(e)
    #             break

    # range(10) | xThreadPoolExecutor(f, 10) | xlist

    # _ = Completions().acreate(
    #     ChatCompletionRequest(messages=[{'role': 'user', 'content': '今天南京天气'}]))
    #
    # # async def main():
    # #     async for i in _:
    # #         yield i
    # print(type(_))
    # for i in async2sync_generator(_):
    #     print(i)

    print(Completions().file_extract('/Users/betterme/PycharmProjects/AI/ChatLLM/data/孙子兵法.pdf'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : embedding
# @Time         : 2023/12/12 10:39
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from openai.types import EmbeddingCreateParams


class EmbeddingRequest(BaseModel):
    model: str = "SentenceTransformer"
    input: Union[str, List[str]] = Field(description="The input to embed.")  # max_length=1000

    user: Optional[str] = Field(default=None)
    encoding_format: Literal["float", "base64"] = 'float'

    model_config = {
        "json_schema_extra": {
            "examples": [
                {
                    "model": "text-embedding-ada-002",
                    "input": "免费GPT就在这 https://api.chatllm.vip",
                }
            ]
        }
    }
